<?php
/* =============================================================================
 * $Revision: 2439 $
 * $Date: 2007-12-20 15:01:14 +0100 (Thu, 20 Dec 2007) $
 * 
 * Vivvo CMS 4.0 
 * Copyright 2005-07 SpoonLabs d.o.o.
 * http://www.spoonlabs.com, All Rights Reserved
 * 
 * Warning: This program is protected by copyright law. Unauthorized
 * reproduction or distribution of this program, or any portion of it, may
 * result in severe civil and criminal penalties, and will be prosecuted to the
 * maximum extent possible under the law. For more information about this
 * script or other scripts see http://www.spoonlabs.com
 * ============================================================================ 
 */

 

/** 
 * Tags object 
 * 
 * @license		http://www.vivvo.net/license.php 
 * @copyright	SpoonLabs 
 * @author		Vedran Martinovic <vmartinovic@spoonlabs.com> 
 * @author		Slobodan Utvic <utvara@spoonlabs.com> 
 * @see		vivvo_post_object 
 * @version		Vivvo Lite - Generic Database Engine
 */
class Tags extends vivvo_post_object {

	/**
	 * id 
	 * Database field type:	int(11) 
	 * Null status:		NO
	 *
	 * @var	integer	$id
	 */
	var $id;
	
	/**
	 * name 
	 * Database field type:	varchar(20) 
	 * Null status:		NO
	 *
	 * @var	string	$name
	 */
	var $name;
	
	var $rating;
	
	/**
	 * status 
	 * Database field type:	enum('0','1') 
	 * Null status:		NO
	 *
	 * @var	string	$status
	 */
	var $status;

	
	var $_sql_table = 'Tags';
	var $fontsize;


	/**
	 * Sets {@link $id}
	 *
	 * @param	integer	$id
	 */
	function set_id($id){
		$this->id = (int) $id;
		return true;
	}

	/**
	 * Sets {@link $name}
	 *
	 * @param	string	$name
	 */
	function set_name($name){
		if ($name != ''){
			$this->name = $name;
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Sets {@link $rating}
	 *
	 * @param	integer	$rating
	 */
	function set_rating($rating){
		$this->rating = $rating;
	}

	/**
	 * Sets {@link $status}
	 *
	 * @param	string	$status
	 */
	function set_status($status){
		if ($status == '1'){
			$this->status = '1';
		}else{
			$this->status = '0';
		}
		return true;
	}

	function set_fontsize($size){
		$this->fontsize = $size;
	}
		


	/** 
	 * Gets $id
	 *
	 * @return	integer
	 */
	function get_id(){
		return $this->id;
	}
	/** 
	 * Gets $name
	 *
	 * @return	string
	 */
	function get_name(){
		return $this->name;
	}
	/** 
	 * Gets $rating
	 *
	 * @return	integer
	 */
	function get_rating(){
		return $this->rating;
	}
	/** 
	 * Gets $status
	 *
	 * @return	string
	 */
	function get_status(){
		return $this->status;
	}
	
	function get_fontsize(){
		return $this->fontsize;
	}
	
	function get_href(){
		if (VIVVO_FRIENDY_URL == 1) {
			if (preg_match('/^[a-z_A-Z0-9\-]+$/', $this->name)){
				return 'tag/' . $this->name;
			}else{
				return 'tag/' . $this->id;
			}
		}else{
			return 'index.php?tag=' . $this->id;
		}
	}
	
	function on_delete(&$post_master){
		
		require_once(dirname(__FILE__) . '/ArticlesTags.class.php');
		$article_tags_list = new ArticlesTags_list($this->_site_manager);
		$article_tags_list->get_all_articles_tags_by_tag_id($this->id);
		$article_tags_list->sql_delete_list($post_master);
		
	}
}

/** 
 * Tags list 
 *
 * @license		http://www.vivvo.net/license.php 
 * @copyright	SpoonLabs 
 * @author		Slobodan Utvic <utvara@spoonlabs.com> 
 * @author		Vedran Martinovic <vmartinovic@spoonlabs.com> 
 * @see			vivvo_db_paged_list
 * @version		Vivvo Lite - Generic Database Engine
 */
class Tags_list extends vivvo_db_paged_list {
	var $_sql_table = 'Tags';
	var $post_object_type = 'Tags';
	
	function _default_query(){
		$this->_query->set_from(VIVVO_DB_PREFIX . 'Tags as t');
		$this->_query->add_fields('COUNT(DISTINCT at.article_id) as rating');
		if (is_array($this->_fields) && !empty($this->_fields)){
			foreach ($this->_fields as $field){
				$this->_query->add_fields('t.' . $field);
			}
		}else{
			$this->_query->add_fields('t.*');
		}
		$this->_query->add_join(' LEFT JOIN ' . VIVVO_DB_PREFIX . 'ArticlesTags as at ON at.tag_id = t.id ','at');
		$this->_query->add_group_by('t.id');
	}
	
	function add_filter($type, $condition = ''){
			
		$condition = secure_sql($condition);
		switch ($type){
			case 'id':
				$this->_query->add_where('(t.id = \'' . $condition . '\')');
			break;
			case 'id_in':
				$condition = secure_sql_in($condition);
				$this->_query->add_where('(t.id IN (' . $condition . '))');
			break;
			case 'name':
				$this->_query->add_where('(t.name = \'' . $condition . '\')');
			break;
			case 'name_array':
				foreach($condition as $k){
					$name_array .= '\''.$k.'\',';
				}
				$name_array = rtrim($name_array,',');
				$this->_query->add_where('(t.name IN (' . $name_array . '))');
			break;
			
			case 'status':
				if ($condition != ''){
					$this->_query->add_where('(t.status = \'' . $condition . '\')');
				}				
			break;
			case 'search_starting_with':
				$condition = str_replace('%', '\%', $condition);
				$this->_query->add_where("t.name LIKE '" . $condition . "%'");
				break;
			case 'status':
				$this->_query->add_where('(t.status = \'' . $condition . '\')');
			break;

		}
	}
	
	/**
	 * Advaced search list generator
	 *
	 * @param	array	$params	Search parameters
	 * @param	string	$order	Order parameters
	 * @param	integer	$limit	Limit
	 * @param	integer	$offset	Offset
	 * @return	array	Array of articles
	 */
	function &search($params, $order='', $direction = 'ascending', $limit =0, $offset =0, $set_list = true){				
		//search_query	
		if (isset($params['search_id'])){
			$this->add_filter('id',$params['search_id']);
			$this->add_generator_option('search_id', $params['search_id']);
		}
		if (isset($params['search_name'])){
			$this->add_filter('name',$params['search_name']);
			$this->add_generator_option('search_name', $params['search_name']);
		}
		if (isset($params['search_status'])){
			$this->add_filter('status',$params['search_status']);
			$this->add_generator_option('search_status', $params['search_status']);
		}
		
		if (isset($params['search_starting_with'])){
			$this->add_filter('search_starting_with',$params['search_starting_with']);
			$this->add_generator_option('search_starting_with', $params['search_starting_with']);
		}
		
		if (isset($params['search_id_in'])){
			$this->add_filter('id_in',$params['search_id_in']);
			$this->add_generator_option('search_id_in', $params['search_id_in']);
		}
		
	    if (isset($params['search_name_array'])){
			$this->add_filter('name_array',$params['search_name_array']);
			$this->add_generator_option('search_name_array', $params['search_name_array']);
		}

			
		// search order //
		$search_direction = ($direction === 'descending') ? ' DESC' : ' ASC';

		switch ($order){
			case 'id':
				$this->_query->add_order('t.id' . $search_direction);
				break;
			case 'name':
				$this->_query->add_order('t.name' . $search_direction);
				break;
			case 'status':
				$this->_query->add_order('t.status' . $search_direction);
				break;

			default:
				$order = 'id';	
				$this->_query->add_order('t.id' . ' DESC');
				break;
		}
		$this->add_order_option($order, $direction);
					
		$limit = (int) $limit;
		$this->_query->set_limit($limit);
		$offset = (int) $offset;
		$this->_query->set_offset($offset);
		$this->_default_query(true);

		if ($set_list){
			$this->set_list();
			return $this->list;
		}
	}

	function &get_tag_by_name($condition){
		$this->_default_query();
		if (is_array($condition)){
			$this->add_filter('name_array' , $condition);
		}else{
			$this->add_filter('name' , $condition);
		}
		
		$this->set_list();
		if (!empty($this->list)){
			return current($this->list);
		}else{
			return false;
		}
	}
	
	function &get_tag_by_id($id){
		$id = (int) $id;
		$this->_default_query();
		$this->add_filter('id' , $id);
		$this->set_list();
		if (!empty($this->list)){
			return current($this->list);
		}else{
			return false;
		}
	}
	
	function &get_all_tags(){
		$this->_default_query();
		
		$this->set_list();
		if (!empty($this->list)){
			return $this->list;
		}else{
			return false;
		}
	}
	
	function &get_orphan_tags($tag_array = array()){
		$this->_default_query();
		if (is_array($tag_array) && !empty($tag_array)){
			$tag_array = implode(',',$tag_array);
			$this->add_filter('id_in',$tag_array);
		}
		$this->_query->add_fields('count( at.tag_id )');
		$this->_query->add_having('COUNT(at.tag_id) = 0');
		$this->set_list();
		if (!empty($this->list)){
			return $this->list;
		}else{
			return false;
		}

	}
	
	function set_fontsize(){
		$min = 0;
		$max = 0;
		if (is_array($this->list) && !empty($this->list)){
			foreach ($this->list as $v){
				$min = min ($min, $v->rating);
				$max = max ($max, $v->rating);
			}
			foreach ($this->list as $k => $v){
				if ($mix == $max){
					$ratio = 2;
				}else{
					$ratio = floor(5 * ($this->list[$k]->rating - $min) / ($max - $min));
				}
				switch ($ratio){
					case 5:
						$fonsize = 24;
						break;
					case 4:
						$fonsize = 18;
						break;
					case 3:
						$fonsize = 14;
						break;
					case 2:
						$fonsize = 12;
						break;
					case 1:
						$fonsize = 10;
						break;
					default:
						$fonsize = 9;
						break;
				}
				$this->list[$k]->set_fontsize($fonsize);
			}
		}
	}
	
	function &get_rated_tags($limit = 0){
		$this->_default_query();
		
		$this->add_filter('status');
		
		$this->_query->add_order('rating DESC');
		
		$this->_query->set_limit($limit);			
		$this->set_list();			
		if (!empty($this->list)){
			$ids = array();
			foreach ($this->list as $tag){
				array_push ($ids, $tag->id);
			}
			$this->_query->reset_query();
			
			$this->_default_query();
			
			$this->add_filter('id_in', implode(',' , $ids));
		
			$this->_query->add_order('name ASC');
			
			$this->_query->set_limit($limit);			
			$this->set_list();			
			
			return $this->list;
		}else{
			return false;
		}
	}
}

?>
