<?php
/* =============================================================================
 * $Revision: 2377 $
 * $Date: 2007-11-07 11:22:24 +0100 (Wed, 07 Nov 2007) $
 * 
 * Vivvo CMS 4.0 
 * Copyright 2005-07 SpoonLabs d.o.o.
 * http://www.spoonlabs.com, All Rights Reserved
 * 
 * Warning: This program is protected by copyright law. Unauthorized
 * reproduction or distribution of this program, or any portion of it, may
 * result in severe civil and criminal penalties, and will be prosecuted to the
 * maximum extent possible under the law. For more information about this
 * script or other scripts see http://www.spoonlabs.com
 * ============================================================================ 
 */

	/**
	 * Module class
	 *
	 * @license		http://www.vivvo.net/license.php
	 * @copyright	SpoonLabs
	 * @package		Vivvo
	 * @subpackage	framework
	 * @version		0.1
	 * @category	CMS
	 * @author		Slobodan Utvic <utvara@spoonlabs.com>
	 * @see			vivvo_object
	 */

	class module extends vivvo_object {		
		/**
		 * Output
		 *
		 * @var string
		 */
		var $output;
		
		/**
		 * Parent template
		 * 
		 * @var template
		 */
		var $_parent_template;
		
		/**
		 * Module identifyer
		 *
		 * @var string
		 */
		var $_name = '';
		
		/**
		 * Default template to process
		 *
		 * @var unknown_type
		 */
		var $_default_template_file = '';
		
		/**
		 * Module configuration params
		 *
		 * @var array
		 */
		var $_configuration = array();
		
		/**
		 * Local template parser
		 *
		 * @var template
		 */
		var $_template;
		
		/**
		 * Cache module output
		 * 
		 * @var boolean
		 */
		var $cache = false;
		
		/**
		 * Cache output string
		 * 
		 * @var string
		 */
		var $cache_output;
		
		/**
		 * Cache manager
		 * 
		 * @var Cache_lite
		 */
		var $_cache_manager;
		
		/**
		 * Cache id MD5 hash of serialized params and class name
		 * 
		 * @var string
		 */
		var $_cache_id;
		
		/**
		 * Create template parser
		 *
		 * @return template
		 */
		function &set_template($params = array()){
			require_once(dirname(__FILE__) . '/vivvo_xml_template.php');

			if (!$this->_parent_template){
				$this->_parent_template =& $this->_site_manager->get_template();
			}
			
			$this->_template = new template($this->_site_manager, $this->_parent_template);
			
			if (!empty($params)){
				
				if (!empty($params['template']) && file_exists(VIVVO_FS_TEMPLATE_ROOT . VIVVO_TEMPLATE_DIR . $params['template'])){
					$this->_template->set_template_file(VIVVO_FS_TEMPLATE_ROOT . VIVVO_TEMPLATE_DIR . $params['template']);
				}elseif (!empty($params['template']) && file_exists(VIVVO_FS_TEMPLATE_ROOT . $params['template'])){
					$this->_template->set_template_file(VIVVO_FS_TEMPLATE_ROOT . $params['template']);
				}elseif(isset($params['template_string'])){
					$this->_template->set_string_template($params['template_string']);
				}else{
					
					$this->_template->set_template_file(VIVVO_FS_TEMPLATE_ROOT . VIVVO_TEMPLATE_DIR . $this->_default_template_file);
				}	
			}else{
				$this->_template->set_template_file(VIVVO_FS_TEMPLATE_ROOT . VIVVO_TEMPLATE_DIR . $this->_default_template_file);
			}

			return $this->_template;
		}
		
		/**
		 * Set parent template
		 *
		 * @param template $parent_template
		 */
		function set_parent_template (&$parent_template){
			$this->_parent_template =& $parent_template;
		}
		
		/**
		 * Save module preferences
		 *
		 * @param vivvo_site $site_manager
		 * @param vivvo_post_master $post_master
		 * @param integer $box_def_id
		 * @param array $prefs
		 * @return mixed on SUCCES true, on ERROR error code
		 */
		function save_preferences (&$site_manager, &$post_master, $box_def_id, $prefs){
			$box_params_list = new box_def_list($site_manager);			
			$box = $box_params_list->get_box_by_id($box_def_id);
			if ($box) {
				$box->set_params('');
				
				if (!empty($prefs) && is_array($prefs)){
					$box->set_params($prefs);
				}
				$post_master->set_data_object($box);
				if (!$post_master->sql_update()){
					// DB insert fail
					return 801;
				}
				return true;
			}else{
				//TODO error code
				return 802;
			}
		}
		
		/**
		 * Set cache manager
		 * 
		 */
		function set_cache_manager(){
			$this->_cache_manager =& $this->_site_manager->get_cache_manager();
		}
		
		/**
		 * Set cache id
		 * 
		 * @param array module load parameters
		 * @return string MD5 hash of serialized params and class name
		 */
		function set_cache_id($params){
			//$this->_cache_id = md5(serialize ($params));
			//TODO: testirati kesiranje
			if (is_array($params)){
				$hash = '';
				foreach($params as $par){
					if (!is_object($par) && !is_array($par)){
						$hash .= $par;
					}elseif (is_array($par)){
						$hash .= $this->set_cache_id($par);
					}
				}
				$this->_cache_id = md5($hash);
			}
			return $this->_cache_id;
		}
		
		function load_cache (){
			if ($this->_cache_manager){
				$cm =& $this->_cache_manager;
				$output = $cm->get($this->_cache_id, get_class($this));
				if ($output){
					$this->cache_output = $output;
					return true;
				}else{
					return false;
				}
			}
		}
		
		/**
		 * Reutrn module template output
		 *
		 * @return string
		 */
		function get_output(){
			if ($this->cache && $this->cache_output){
				return $this->cache_output;
			}else{
				if (is_a($this->_template, 'template')){
					$outupt = $this->_template->get_output();
				}else {
					$outupt = $this->output;
				}
				if ($this->_cache_manager && $this->cache){	
					$this->_cache_manager->save($outupt, $this->_cache_id, get_class($this));
				}
				return $outupt;
			}
		}
		
		/**
		 * Load settings for module
		 *
		 */
		function set_configuration(){
			if (!is_a($this->_site_manager, 'vivvo_site')) $this->set_site_manager();
			$config = $this->_site_manager->get_configuration();
			$this->_configuration = $config->get_configuration($this->_name);
		}
		
		function generate_output(){
			$this->output = '';
		}
		
		/**
		 * Default module params
		 * 
		 * @return array default params
		 */
		function default_params(){
			return array();
		}
		
		/**
		 * Generic module counstructor
		 *
		 * @param vivvo_site $site_manager
		 * @param array $params
		 * @param boolean $cache 				force cache drop
		 * @return module
		 */
		function module(&$site_manager, $params, $parent_template = NULL, $cache = true, $output = true){
			$this->set_site_manager($site_manager);
			$this->set_cache_manager();
			
			if ($parent_template) $this->set_parent_template($parent_template);
			
			if ($this->cache && $cache){
				$this->set_cache_id($params);
				if (!$this->load_cache()){
					if ($output) $this->generate_output($params);
				}
			}elseif ($params['cache'] == 1){
				$this->set_cache_id($params);
				$this->cache = true;
				if (!$this->load_cache()){
					if ($output) $this->generate_output($params);
				}
			}else{
				if ($output) $this->generate_output($params);
			}
		}
		
		function __destruct () {
			parent::__destruct ();
			if (!empty($this->_template)){
				$this->_template->__destruct();
				unset($this->_template);
				
			}
			unset($this->_configuration);
			if (!empty($this->_parent_template)) unset($this->_parent_template);
			if (!empty($this->_cache_manager)) unset($this->_cache_manager);
		}
	}
	
?>