<?php	
/* =============================================================================
 * $Revision: 2480 $
 * $Date: 2008-01-04 13:05:48 +0100 (Fri, 04 Jan 2008) $
 * 
 * Vivvo CMS 4.0 
 * Copyright 2005-07 SpoonLabs d.o.o.
 * http://www.spoonlabs.com, All Rights Reserved
 * 
 * Warning: This program is protected by copyright law. Unauthorized
 * reproduction or distribution of this program, or any portion of it, may
 * result in severe civil and criminal penalties, and will be prosecuted to the
 * maximum extent possible under the law. For more information about this
 * script or other scripts see http://www.spoonlabs.com
 * ============================================================================ 
 */

	/**
	 * vivvo_db_list class
	 *
	 * @license		http://www.vivvo.net/license.php
	 * @copyright	SpoonLabs
	 * @package		Vivvo
	 * @subpackage	framework
	 * @version		0.1
	 * @category	CMS
	 * @author		Slobodan Utvic <utvara@spoonlabs.com>
	 */
	class vivvo_db_list extends vivvo_object {
				
		/**
		 * MDB2_Driver_Common
		 *
		 * @var MDB2_Driver_Common
		 */
		var $_db;
		
		var $_sql_table = '';
		
		/**
		 * query builder
		 *
		 * @var vivvo_query_builder
		 */
		var $_query;
		
		var $_fields = '*';
		
		var $list = array();
		var $post_object_type;
		var $id_key = 'id';
		
		function set_db(){
			$this->_db =& $this->_site_manager->_db;
		}
		
		function set_query(){
			$this->_query = new vivvo_query_builder();
		}
		
		function _default_query(){
			$this->_query->set_from(VIVVO_DB_PREFIX . $this->_sql_table);
			if (is_array($this->_fields) && !empty($this->_fields)){
				foreach ($this->_fields as $field){
					$this->_query->add_fields($field);
				}
			}else{
				$this->_query->add_fields('*');
			}	
		}
		
		/**
		 * Delete all list objects from database
		 *
		 * @param vivvo_post_master $post_master
		 * @param array $restiction		Restriction ids
		 * 
		 * @return bool	succes
		 */
		function sql_delete_list(&$post_master, $restriction = array(), $all_matching = false){
			$result = true;
			$ids = $this->get_list_ids();

			if ($ids && is_array($restriction) && !empty($restriction)){
				$ids = array_intersect($ids, $restriction);
			}
	
			if (!empty($ids)){
				foreach ($ids as $id){
					$where[] = $this->_site_manager->_db->quote($id, 'integer');
					$this->list[$id]->on_delete($post_master);
					
				}

				$where = $this->id_key . ' IN ('. implode(',' , $where) .')';
				$result = $post_master->sql_delete_list($this->_sql_table, $where);

				foreach ($ids as $id){
					unset($this->list[$id]);
				}
			}elseif ($all_matching === true){
				$where = str_replace('WHERE', ' ', $this->_query->get_where());
				$from = str_replace('FROM', ' ', $this->_query->get_from());
				$result = $post_master->sql_delete_list($this->_sql_table, $where, $from);
			}
			return $result;
		}
		
		/**
		 * Insert all objects from list to databas
		 *
		 * @param vivvo_post_master $post_master
		 * 
		 * @return bool	succes
		 */
		function sql_insert_list(&$post_master){
			$result = true;
			if (!empty($this->list)){
				foreach ($this->list as $k => $v){
					$result = $result && $post_master->sql_insert($this->list[$k]);
				}
			}
			return $result;
		}

		/**
		 * Insert all objects from list to databas
		 *
		 * @param vivvo_post_master $post_master
		 * @param array $params Params to change assoc array
		 * @param array $restiction		Restriction ids
		 * 
		 * @return bool	succes
		 */
		function sql_update_list(&$post_master, $params, $restriction = NULL, $all_matching = false){
			$result = true;
			$ids = $this->get_list_ids();
			if ($ids && is_array($restriction) && !empty($restriction)){
				$ids = array_intersect($ids, $restriction);
			}
			if ($ids){
				foreach ($ids as $id){
					$where[] = $this->_site_manager->_db->quote($id);
				}

				$where = $this->id_key . ' IN ('. implode(',' , $where) .')';
				$result = $post_master->sql_update_list(VIVVO_DB_PREFIX . $this->_sql_table, $params, $where);
			}elseif ($all_matching === true){
				$where = str_replace('WHERE', ' ', $this->_query->get_where());
				$from = str_replace('FROM', ' ', $this->_query->get_from());
				$result = $post_master->sql_update_list(VIVVO_DB_PREFIX . $this->_sql_table, $params, $where, $from);
			}
			return $result;
		}
		
		function &get_item_by_property($prop, $value, $index = 0){
			$curent_index = 0;

			if (is_array($this->list) && !empty($this->list)){
				$keys = $this->get_list_ids();
				foreach($keys as $key){
					if ($this->list[$key]->__get($prop) == $value){
						if ($curent_index == $index){
							return $this->list[$key];
						}
						$curent_index++;
					}
				}
			}
			return false;
		}
		
		function get_sublist($ids){
			if (is_array($ids) && !empty($ids)){
				$array = array();
				foreach ($ids as $id){
					if (is_array($this->list) && key_exists($id,$this->list)){
						$array[$id] =& $this->list[$id];
					}
				}
				return $array;
			}
			return false;
		}
		
		function get_ids_by_property($prop, $value){
			if (is_array($this->list) && !empty($this->list)){
				$array = array();
				$keys = $this->get_list_ids();
				foreach($keys as $key){
					if (is_array($value)){
						if (in_array($this->list[$key]->__get($prop), $value)){
							$array[] = $key;
						}
					}else{
						if ($this->list[$key]->__get($prop) == $value){
							$array[] = $key;
						}	
					}
				}
				return $array;
			}
			return false;
		}
		
		function &get_item_list_by_property($prop, $value){
			if (is_array($this->list) && !empty($this->list)){
				$array = $this->get_ids_by_property($prop, $value);
				return $this->get_sublist($array);
			}
			return false;
		}
		
		function get_property_list($prop_name){
			if (is_array($this->list) && !empty($this->list)){
				$array = array();
				$keys = $this->get_list_ids();
				foreach($keys as $key){
					$array[] = $this->list[$key]->__get($prop_name);
				}
				return $array;
			}
			return false;
		}
		
		function get_total_count(){
			$query = $this->_query->get_total_count_query();
			$res =& $this->_db->query($query);
			if (!is_a($res, 'mdb2_error')){
				$sum = 0;
				$num = 0;
				while ($count = $res->fetchOne()){
					$sum += $count;
					$num++;
				}
				$res->free();
				if ($num > 1){
					return $num;	
				}else{
					return $sum;
				}
			}else {
				return false;
			}
		}
		
		function get_list_ids(){
			if (is_array($this->list)){
				return array_keys($this->list);
			}else{
				return false;
			}
		}
		
		function get_total_number_of_fields(){
			$res =& $this->_db->query('SELECT count(*) FROM ' . VIVVO_DB_PREFIX . $this->_sql_table);
			if (!is_a($res, 'mdb2_error')){
				$sum = 0;
				while ($count = $res->fetchOne()){
					$sum += $count;				
				}
				$res->free();
				return $sum;
			}else {
				return false;
			}	
		}
		
		function get_count($search_params = false){
			if ($search_params !== false){
				$this->search($search_params, '', '', 0, 0, false);
			}else{
				$this->_default_query();
			}
			
			return $this->get_total_count();	
		}
		
		function get_max($field, $search_params = false){
			if ($search_params !== false){
				$this->search($search_params, '', '', 0, 0, false);
			}else{
				$this->_default_query();
			}
			
			
			$query = $this->_query->get_max_query($field);

			$res =& $this->_db->query($query);
			if (!is_a($res, 'mdb2_error')){
				return $res->fetchOne();
			}else {
				return false;
			}
		}
		
		function get_min($field, $search_params = false){
			if ($search_params !== false){
				$this->search($search_params, '', '', 0, 0, false);
			}else{
				$this->_default_query();
			}
			
			$query = $this->_query->get_min_query($field);
			$res =& $this->_db->query($query);
			if (!is_a($res, 'mdb2_error')){
				return $res->fetchOne();
			}else {
				return false;
			}
		}
		
		function get_sum($field, $search_params = false){
			if ($search_params !== false){
				$this->search($search_params, '', '', 0, 0, false);
			}else{
				$this->_default_query();
			}
			
			$query = $this->_query->get_sum_query($field);
			$res =& $this->_db->query($query);
			if (!is_a($res, 'mdb2_error')){
				return $res->fetchOne();
			}else {
				return false;
			}
		}
		
		function prepare_for_xml($data){
//			return $data;
			return str_replace(array('&', '"', "'", '<', '>'), array('&amp;', '&quot;', '&apos;', '&lt;', '&gt;'), $data);
		}
		
		function remove_xml_escapes($data){
//			return $data;
			return str_replace(array('&amp;', '&quot;', '&apos;', '&lt;', '&gt;'), array('&', '"', "'", '<', '>'), $data);
		}
		
		function set_list(){
			$this->list = array();
			if (!empty($this->post_object_type) && class_exists($this->post_object_type)){
				$query = $this->_query->get_query();
				if ($query != ''){					
					$res =& $this->_db->query($query);
					if (!is_a($res, 'mdb2_error')){
						$class = $this->post_object_type;
						while (($row = $res->fetchRow(MDB2_FETCHMODE_ASSOC))){	
							if ($this->_site_manager->_object_cache_manager->is_cached($class, $row[$this->id_key])){

								$this->list[$row[$this->id_key]] =& $this->_site_manager->_object_cache_manager->retrive($class, $row[$this->id_key]);
								$this->list[$row[$this->id_key]]->populate($row);
								
							}else{
								$this->_site_manager->_object_cache_manager->add($class, $row[$this->id_key], new $class($this->_site_manager, $row));
								$this->list[$row[$this->id_key]] =& $this->_site_manager->_object_cache_manager->retrive($class, $row[$this->id_key]);
							}
							
						}
						$res->free();
					}else{
						if (isset($_GET['debug'])) echo "<hr/>query error: $query \n<hr/>\n";
					}
				}else{
					if (isset($_GET['debug'])) echo "bbbbb";
				}
			}else{
				if (isset($_GET['debug'])) echo "cccc";
			}
		}
		
		function generic_add_filter($params, $prefix = ''){
			if (is_array($params) && !empty($params)){
				$keys = array_keys($params);
				foreach ($keys as $k){
					if (preg_match('/^search_field_(.*)_(lt|gt|eq|neq|in|notin|between|notnull)$/', $k, $arr)){
						$condition = secure_sql($params[$k]);
						switch ($arr[2]){
							case 'lt':
								$this->_query->add_where('(' . $prefix . $arr[1] . ' < \'' . $condition . '\')');
							break;
							case 'gt':
								$this->_query->add_where('(' . $prefix . $arr[1] . ' > \'' . $condition . '\')');
							break;
							case 'eq':
								$this->_query->add_where('(' . $prefix . $arr[1] . ' = \'' . $condition . '\')');
							break;
							case 'neq':
								$this->_query->add_where('(' . $prefix . $arr[1] . ' != \'' . $condition . '\')');
							break;
							case 'in':
								$condition = secure_sql_in($condition);
								$this->_query->add_where('(' . $prefix . $arr[1] . ' IN ' . $condition . ')');
							break;
							case 'notin':
								$condition = secure_sql_in($condition);
								$this->_query->add_where('(' . $prefix . $arr[1] . ' NOT IN ' . $condition . ')');
							break;
							case 'notnull':
								$this->_query->add_where('(' . $prefix . $arr[1] . ' IS NOT NULL )');
							break;
							case 'between':
								$between = explode(',', $condition);
								if (is_array ($between) && count($between) == 2){
									$this->_query->add_where('(' . $prefix . $arr[1] . ' BETWEEN \'' . $between[0] . '\' AND \'' . $between[1] . '\')');
								}
							break;
						}
					}
				}
			}
		}
		
		/**
		 * Generic serach params filter
		 *
		 * @param vivvo_site $sm
		 * @param string $table
		 * @param array $in_params
		 */
		function generic_get_search_params(&$sm, $table, $in_params){
			if (is_array($in_params) && !empty($in_params)){
				$dbm =& $sm->get_db_manager();
				$fields = $dbm->get_table_fields($table);
				if (!empty($fields)){
					$our_params = array();
					$keys = array_keys($in_params);
					$field_keys = implode('|', $fields);

					foreach ($keys as $k){
						if (preg_match('/^search_field_(' . $field_keys . ')_(lt|gt|eq|neq|in|notin|between|notnull)$/', $k)){
							$our_params[$k] = $in_params[$k];
						}
					}
					return $our_params;
				}
			}
			return false;
		}
		
		/**
		 * Generic sort
		 *
		 * @param string $prefix
		 * @param string $sort
		 * @param string $direction
		 */
		function generic_sort($prefix, $sort, $direction){
			if (!empty($sort)){
				$dbm =& $this->_site_manager->get_db_manager();
				$fields = $dbm->get_table_fields(VIVVO_DB_PREFIX . $this->_sql_table);
				if (!empty($fields)){
					if (in_array($sort, $fields)){
						$this->_query->add_order($prefix . $sort . ' ' . $direction);
					}
					return true;
				}
			}
			return false;
		}
		
		function reset_list_query(){
			$this->_query->reset_query();
		}
		
		function vivvo_db_list(&$site_manager, $fields = ''){			
			if ($site_manager){
				$this->set_site_manager($site_manager);
				$this->set_db();
				
				$this->set_query();
			}			
			
			if (!empty($fields)){
				$fields = explode(',', $fields);
				if (!in_array($this->id_key , $fields)){
					$fields[] = $this->id_key;
				}
				$this->_fields = $fields;
			}else{
				$this->_fields = '.*';
			}
		}
		
		
		function __destruct() {
			parent::__destruct();
			if (!empty($this->_db)) unset($this->_db);
			if (!empty($this->_query)) unset($this->_query);
			/*
			if (!empty($this->list)){
				$keys = array_keys($this->list);
				foreach($keys as $k){
					unset($this->list[$k]);
				}
				unset($this->list);
			}
			*/
		}
	}
?>