<?php
/* =============================================================================
 * $Revision: 2577 $
 * $Date: 2008-02-22 15:49:54 +0100 (Fri, 22 Feb 2008) $
 * 
 * Vivvo CMS 4.0 
 * Copyright 2005-07 SpoonLabs d.o.o.
 * http://www.spoonlabs.com, All Rights Reserved
 * 
 * Warning: This program is protected by copyright law. Unauthorized
 * reproduction or distribution of this program, or any portion of it, may
 * result in severe civil and criminal penalties, and will be prosecuted to the
 * maximum extent possible under the law. For more information about this
 * script or other scripts see http://www.spoonlabs.com
 * ============================================================================ 
 */

	
/**
 * Vivvo CMS
 *
 * LICENSE: 
 *
 * @package    Vivvo
 * @subpackage plugin
 * @author     Slobodan Utvic <utvara@spoonlabs.com>
 * @copyright  Spoonlbas
 * @license    http://www.word-press.net/license.php
 * @version    0.1
 * @category   CMS
 */

	/**
	 * Plugin manager class
	 *
	 * @license		http://www.vivvo.net/license.php
	 * @copyright	SpoonLabs
	 * @package		Vivvo
	 * @subpackage	plugin
	 * @version		0.1
	 * @category	CMS
	 * @author		Slobodan Utvic <utvara@spoonlabs.com>
	 */
	
	Class plugin_manager extends vivvo_object{
		/**
		 * Plugin direcory
		 *
		 * @var string
		 */
		
		var $plugin_dir;
		
		/**
		 * Plugins avalable
		 *
		 * @var array
		 */
		var $plugins = array();

		/**
		 * FS plugins
		 * 
		 * @var array
		 */
		var $fs_plugins;

		/**
		 * Reads configuration from info.php files
		 * in all plugins directory like bm, newsletter, poll, sitemap, ...
		 * 
		 */
		function set_fs_plugins(){
			global $plugin_definition;
			if ($handle = opendir($this->plugin_dir)) {
				while ((false !== ($file = readdir($handle))) && ($temp == 0)) {
					if ($file[0] != '.'){
						$file = $this->plugin_dir . DIRECTORY_SEPARATOR . $file;
						if (is_dir($file) && is_file ($file . DIRECTORY_SEPARATOR . 'info.php')){
							include($file . DIRECTORY_SEPARATOR . 'info.php');
							$this->fs_plugins[$plugin_definition['href']] = $plugin_definition; 
						}
					}
				}
			}
		} 
		
		/**
		 * Reads configuration from configuration table
		 *
		 */
		
		function set_plugins(){
			$config =& $this->_site_manager->get_configuration();
			$plugin_list = $config->get_configuration('plugins');
			if (is_array($plugin_list) && !empty($plugin_list)){
				foreach ($plugin_list as $pld){
					$plugin_def = $config->get_configuration($pld['variable_value']);
					$this->plugins[$pld['variable_value']] = array();
					if (is_array($plugin_def) && !empty($plugin_def)){
						foreach ($plugin_def as $pd){
							$this->plugins[$pld['variable_value']][$pd['variable_name']] = $pd['variable_value'];
						}
					}
				}
			}
			asort($this->plugins);
		}

		function check_fs ($plugin_name){
			global $plugin_file_list;
			$files_def = $this->plugin_dir . '/'. $plugin_name .'/plugin_fs.php';
			if (file_exists($files_def)){
				require($files_def);
				if (is_array($plugin_file_list) && !empty($plugin_file_list)){
					foreach ($plugin_file_list as $filename => $checksum){
						if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $filename)){
							if (md5_file($this->plugin_dir . '/'. $plugin_name . '/' . $filename) != $checksum){
								$this->set_error_code(5201);
								echo $this->plugin_dir . '/'. $plugin_name . '/' . $filename;
								return false;
							}
						}else{
							$this->set_error_code(5202);
							return false;
						}
					}
				}
			}else {
				$this->set_error_code(5203);
				return false;
			}		
			return true;
		}
		
		function check_fs_privilegies ($plugin_name){
			global $plugin_file_list;
			$files_def = $this->plugin_dir . '/'. $plugin_name .'/plugin_fs_privilegies.php';
			if (file_exists($files_def)){
				require($files_def);
				if (is_array($plugin_file_list) && !empty($plugin_file_list)){
					foreach ($plugin_file_list as $filename => $mask){
						if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $filename)){
							if ($mask % 2 !== 0){
								if (!is_executable($this->plugin_dir . '/'. $plugin_name . '/' . $filename)){
									return false;
								}
								$mask--;
							}
							$mask = $mask / 2;
							
							if ($mask % 2 !== 0){
								if (!is_writeable($this->plugin_dir . '/'. $plugin_name . '/' . $filename)){
									return false;
								}
								$mask--;
							}
							$mask = $mask / 2;
							
							if ($mask % 2 !== 0){
								if (!is_readable($this->plugin_dir . '/'. $plugin_name . '/' . $filename)){
									return false;
								}
							}
						}else{
							$this->set_error_code(5204);
							return false;
						}
					}
				}
			}else {
				$this->set_error_code(5205);
				return false;
			}		
			return true;
		}
		
		function install($plugin_name){
			$this->set_fs_plugins();
			if (is_array($this->fs_plugins) && key_exists($plugin_name, $this->fs_plugins)){
				//STEP 1: Check FS
				
				$this->check_fs($plugin_name);
				
				//STEP 2: Check FS permisions
				
				$this->check_fs_privilegies($plugin_name);
				
				//STEP 3: Create DB tables			
				
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					call_user_func(array($plugin_obj,'install'), $this->_site_manager);
				}
			}
		}
		
		function uninstall($plugin_name){
			$this->set_fs_plugins();
			if (is_array($this->fs_plugins) && key_exists($plugin_name, $this->fs_plugins)){
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					call_user_func(array($plugin_obj,'uninstall'), $this->_site_manager);
				}
			}else{
				$this->set_error_code(5206);
				return 888;
			}
		}
		
		function get_all_plugins() {
			$plugin_array = array();
			$config =& $this->_site_manager->get_configuration();
			$this->set_fs_plugins();
			if (is_array($this->fs_plugins) && !empty($this->fs_plugins)){
				foreach ($this->fs_plugins as $key => $pd){
					if (!key_exists($key, $this->plugins)){
						$plugin = $pd;
						if (is_array($plugin) && !empty($plugin)){
							$plugin_array[$key] = array();
							foreach ($plugin as $k => $property){
								$plugin_array[$key][$k] = $property;
							}
							$plugin_array[$key]['href'] = $key;
							$plugin_array[$key]['installed'] = 0;
							if (file_exists($this->plugin_dir . "/{$key}/description.html")){
								$plugin_array[$key]['description'] = file_get_contents($this->plugin_dir . "/{$key}/description.html");
							}
						}
					}
				}
			}
			
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){					
					$plugin = $config->get_configuration($key);
					if (is_array($plugin) && !empty($plugin)){
						$plugin_array[$key] = array();
						foreach ($plugin as $property){
							$plugin_array[$key][$property['variable_name']] = $property['variable_value'];
						}
						$plugin_array[$key]['href'] = $key;
						$plugin_array[$key]['installed'] = 1;
						if (file_exists($this->plugin_dir . "/{$key}/description.html")){
							$plugin_array[$key]['description'] = file_get_contents($this->plugin_dir . "/{$key}/description.html");
						}
					}
				}
			}
			ksort($plugin_array);
			return $plugin_array;
		}
		
		function get_xml(){
			$xml = '<v3:plugin_list>';
			$config =& $this->_site_manager->get_configuration();
			$this->set_fs_plugins();
			if (is_array($this->fs_plugins) && !empty($this->fs_plugins)){
				foreach ($this->fs_plugins as $key => $pd){
					if (!key_exists($key, $this->plugins)){
						$plugin = $pd;
						$xml .= "<v3:plugin name=\"$key\">";
						if (is_array($plugin) && !empty($plugin)){
							foreach ($plugin as $k => $property){
								$xml .= "<v3:param name=\"" . $k . "\" value=\"" . $property . "\" />";
							}
						}												
						$xml .= "<v3:param name=\"installed\" value=\"0\" />";
						if (file_exists($this->plugin_dir . "/{$key}/description.html")){
							$xml .= '<v3:description>' .file_get_contents($this->plugin_dir . "/{$key}/description.html") . '</v3:description>';
						}
						$xml .= "</v3:plugin>";
					}
				}
			}
			
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){					
					$plugin = $config->get_configuration($key);
					$xml .= "<v3:plugin name=\"$key\">";
					if (is_array($plugin) && !empty($plugin)){
						foreach ($plugin as $property){
							$xml .= "<v3:param name=\"" . $property['variable_name'] . "\" value=\"" . $property['variable_value'] . "\" />";
						}
					}
					$xml .= "<v3:param name=\"installed\" value=\"1\" />";
					if (file_exists($this->plugin_dir . "/{$key}/description.html")){
						$xml .= '<v3:description>' .file_get_contents($this->plugin_dir . "/{$key}/description.html") . '</v3:description>';
					}
					$xml .= "</v3:plugin>";
				}
			}
			$xml .= '</v3:plugin_list>';
			return $xml;
		}
				
		/**
		 * Load admin javaScript 
		 */
		function load_admin_js (){
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){
					if (file_exists($this->plugin_dir . '/'. $key . '/' . $key . '_definition.class.php')){
						require_once($this->plugin_dir . '/'.  $key . '/' . $key . '_definition.class.php');
						$plugin_obj = $key . '_definition';
						call_user_func(array($plugin_obj,'load_admin_js'), $this->_site_manager->get_header_manager());
					}
				}
			}
		}
		
		function load_admin_css (){
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){
					if (file_exists($this->plugin_dir . '/'. $key . '/' . $key . '_definition.class.php')){
						require_once($this->plugin_dir . '/'.  $key . '/' . $key . '_definition.class.php');
						$plugin_obj = $key . '_definition';
						call_user_func(array($plugin_obj,'load_admin_css'), $this->_site_manager->get_header_manager());
					}
				}
			}
		}
		
		function get_plugin_menu_item (){
			$plugin_array = array();
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){
					if (file_exists($this->plugin_dir . '/'. $key . '/' . $key . '_definition.class.php')){
						require_once($this->plugin_dir . '/'.  $key . '/' . $key . '_definition.class.php');
						$plugin_obj = $key . '_definition';
						if (call_user_func(array($plugin_obj,'get_plugin_menu_item'))){
							$plugin_array[] = call_user_func(array($plugin_obj,'get_plugin_menu_item'));
						}
					}
				}
			}
			ksort($plugin_array);
			return $plugin_array;
		}
		
		function get_local_menu () {
			$menu = '';
			if (is_array($this->plugins) && !empty($this->plugins)){
				foreach ($this->plugins as $key => $pd){
					if (file_exists($this->plugin_dir . '/'. $key . '/' . $key . '_definition.class.php')){
						require_once($this->plugin_dir . '/'.  $key . '/' . $key . '_definition.class.php');
						$plugin_obj = $key . '_definition';
						$menu .= call_user_func(array($plugin_obj,'get_local_menu'));
					}
				}
			}
			return $menu;
		}
		
		function url_handler($plugin_name, $url_array){
			$parsed_url = false;

			if (is_array($this->plugins) && key_exists($plugin_name, $this->plugins)){
				
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					$parsed_url = call_user_func(array($plugin_obj,'url_handler'), $url_array);
				}
			}
			return $parsed_url;
		}
		
		function get_content($plugin_name){
			$content = false;
			if (is_array($this->plugins) && key_exists($plugin_name, $this->plugins)){
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					$content = call_user_func(array($plugin_obj, 'get_content'), $this->_site_manager);
				}
			}
			return $content;
		}
		
		function get_admin_content($plugin_name){
			$content = false;
			if (is_array($this->plugins) && key_exists($plugin_name, $this->plugins)){
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					$content = call_user_func(array($plugin_obj, 'get_admin_content'), $this->_site_manager);
				}
			}
			return $content;
		}
		
		function &get_plugin_definiton ($plugin_name){
			if (is_array($this->plugins) && key_exists($plugin_name, $this->plugins)){
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					if (class_exists($plugin_obj)){
						return new $plugin_obj ($this->_site_manager);
					}else{
						return false;
					}
				}
			}
			return false;
		}
		
		function &get_fs_plugin_definiton ($plugin_name){
			if (is_array($this->fs_plugins) && key_exists($plugin_name, $this->fs_plugins)){
				if (file_exists($this->plugin_dir . '/'. $plugin_name . '/' . $plugin_name . '_definition.class.php')){
					require_once($this->plugin_dir . '/'.  $plugin_name . '/' . $plugin_name . '_definition.class.php');
					$plugin_obj = $plugin_name . '_definition';
					if (class_exists($plugin_obj)){
						return new $plugin_obj ($this->_site_manager);
					}else{
						return false;
					}
				}
			}
			return false;
		}
		
		function load_lang (){
			$lang =& $this->_site_manager->get_lang();
			if (is_array($this->plugins)){
				foreach ($this->plugins as $key => $pd){
					$lang->load_lang(VIVVO_DEFAULT_LANG, $this->plugins[$key]['root_dir'] . 'lang/');
				}
			}
		}
		
		
		function register($plugin_name, $root_dir, $name){
			$configuration =& $this->_site_manager->get_configuration();
			$configuration->add_conf('name', '', $plugin_name, 'plugins');
			$configuration->add_conf('name', '', $name, $plugin_name);
			$configuration->add_conf('root_dir', '', $root_dir, $plugin_name);
			$configuration->add_conf('installed', '', '1', $plugin_name);
		}
		
		function unregister($plugin_name){
			$configuration =& $this->_site_manager->get_configuration();
			$configuration->remove_from_module('plugins', 'name', $plugin_name);
			$configuration->remove_module($plugin_name);
		}
		
		/**
		 * Constuctor
		 *
		 * @return plugin_manager
		 */

		function plugin_manager(&$site_manager){
			$this->set_site_manager($site_manager);
			$this->plugin_dir = VIVVO_FS_PLUGIN_ROOT . 'plugins';
			$this->set_plugins();
		}
	}

	/**
	 * Plugin class
	 *
	 * @license		http://www.vivvo.net/license.php
	 * @copyright	SpoonLabs
	 * @package		Vivvo
	 * @subpackage	plugin
	 * @version		0.1
	 * @category	CMS
	 * @author		Slobodan Utvic <utvara@spoonlabs.com>
	 */
	
	Class plugin extends vivvo_object{
		/**
		 * Plugin identifyer
		 *
		 * @var string
		 */
		var $plugin_name;
		
		/**
		 * User firendly plugin name
		 *
		 * @var string
		 */
		var $plugin_menu_name;
		
		/**
		 * Plugin definition file location
		 *
		 * @var string
		 */
		var $definition_file;
		
		function get_root_dir(){
			$plugin_root_dir = dirname($this->definition_file) . '/';
			$plugin_root_dir = substr($plugin_root_dir, strlen(VIVVO_FS_ROOT));
			$plugin_root_dir = str_replace('\\', '/', $plugin_root_dir);
			return $plugin_root_dir;
		}
		
		/**
		 * Plugin install procedure
		 *
		 * @param	vivvo_site	$site_manager
		 * @param	integer		$step
		 */
		function install(&$site_manager, $step = 1){
			$template =& $site_manager->get_template();
			
			$template->assign('PLUGIN_ROOT', $this->get_root_dir());
			$template->assign('PLUG_IN_NAME', $this->plugin_menu_name);
			$template->assign('PLUG_IN_HREF', $this->plugin_name);

			$content_template =& new template($site_manager, &$template);
			
			if ($step == 2){
				$content_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/install/step_2.xml');
			}else{
				$content_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/install/step_1.xml');
			}
		
			$template->assign('content', $content_template->get_output());
			$template->assign('step', strval($step));
				
			$search_template =& new template($site_manager, $template);
			$search_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/install/tabs.xml');
	
			$template->assign('left', $search_template->get_output());
		}
		
		/**
		 * Uninstall Plugin
		 *
		 * @param vivvo_site 	$site_manager
		 * @param integer		$step
		 */
		function uninstall(&$site_manager, $step = 1){
			$template =& $site_manager->get_template();
			
			$template->assign('PLUGIN_ROOT', $this->get_root_dir());
			$template->assign('PLUG_IN_NAME', $this->plugin_menu_name);
			$template->assign('PLUG_IN_HREF', $this->plugin_name);
			
			$content_template =& new template($site_manager, $template);
			
			if ($step == 2){
				$content_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/uninstall/step_2.xml');;
			}else{
				$content_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/uninstall/step_1.xml');
			}
			
			$template->assign('content', $content_template->get_output());
			$template->assign('step', strval($step));
			
			$search_template =& new template($site_manager, $template);
			$search_template->set_template_file(VIVVO_FS_ROOT . VIVVO_FS_ADMIN_DIR . 'templates/plugins/uninstall/tabs.xml');
	
			$template->assign('left', $search_template->get_output());
		}
		
		/**
		 * Analyze local part of URL
		 *
		 * @return mixed array with sorted data|false on fail
		 */
		function url_handler($url_array){
			return false;
		}
		
		/**
		 * Load admin javaScript 
		 * 
		 */
		function load_admin_js (){
			return false;
		}
		
		function load_admin_css (&$header){

		}
		
		/**
		 * Get plugins page left nav menu item definition
		 *
		 * @return array|string Menu item definition
		 */
		function get_plugin_menu_item (){
			return '';
		}
		
		function get_local_menu () {
			return '';
		}
		
		/**
		 * Load frontend content
		 * 
		 * @param vivvo_lite_site $site_manager
		 * @return string|boolean Template output
		 */
		function get_content(&$site_manager){
			return false;
		}
		
		function load_admin_lang(){
			$pm =& $this->_site_manager->get_plugin_manager();
			$lang =& $this->_site_manager->get_lang();
			$lang->load_lang(VIVVO_ADMIN_LANG, $pm->plugins[$this->plugin_name]['root_dir'] . 'admin/lang/');
		}
		
		function load_lang(){
			$pm =& $this->_site_manager->get_plugin_manager();
			$lang =& $this->_site_manager->get_lang();
			$lang->load_lang(VIVVO_ADMIN_LANG, $pm->plugins[$this->plugin_name]['root_dir'] . 'lang/');
		}
		
		/**
		 * Load administration content
		 * 
		 * @param vivvo_lite_site $sm
		 */
		function get_admin_content (&$sm){
			header("Location: " . VIVVO_URL . VIVVO_FS_ADMIN_DIR . 'plugins.php');
		}
	}
	
	/**
	 * Plugin_module class
	 *
	 * @license		http://www.vivvo.net/license.php
	 * @copyright	SpoonLabs
	 * @package		Vivvo
	 * @subpackage	plugin
	 * @version		0.1
	 * @category	CMS
	 * @author		Slobodan Utvic <utvara@spoonlabs.com>
	 */
	
	Class plugin_module extends module{
		/**
		 * Plugin identifyer
		 *
		 * @var string
		 */
		var $plugin_name;

		function set_lang(){
			//$lang =& $this->_site_manager->get_lang();
			//TODO: plugin lang;
			//$lang->load_lang(VIVVO_LANG, $this->get_configuration('root_dir') . '/lang/', $this->plugin_name);
		}
						
		/**
		 * Assign plugin configuration
		 *
		 * @param array $config
		 * @return boolean
		 */
		function set_configuration($config = ''){
			if (is_array($config)){
				$this->configuration = $config;
				return true;
			}else{
				if (is_a($this->_site_manager, 'vivvo_site')){					
					$config =& $this->_site_manager->get_configuration();
					$this->_configuration = $config->get_configuration_property_list($this->plugin_name);
					return true;
				}else{
					return false;
				}
			}
		}
		
		/**
		 * Get configuration
		 *
		 * @param string $param_name
		 * @return mixed
		 */
		function get_configuration ($param_name){
			if (is_array($this->_configuration) && key_exists($param_name, $this->_configuration)){
				return $this->_configuration[$param_name];
			}else{
				return false;
			}
		}
	}
?>
